@@ -2,7 +2,7 @@
 ### Fit a semiparametric bernoulli regression model.
 ###
 ### Copyright: Alejandro Jara Vallejos, 2006
-### Last modification: 21-08-2006.
+### Last modification: 22-12-2006.
 ###
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the GNU General Public License as published by
@@ -43,8 +43,8 @@ function(formula,
          state,
          status,
          misc=NULL,
-         data,
-         na.action) 
+         data=sys.frame(sys.parent()),
+         na.action=na.fail) 
 {
          #########################################################################################
          # call parameters
@@ -66,7 +66,50 @@ function(formula,
 	 nrec<-length(yobs)
 	 x<-as.matrix(model.matrix(formula))
 	 p<-dim(x)[2]
-	 
+
+         #########################################################################################
+         # Elements for Pseudo Countour Probabilities' computation
+         #########################################################################################
+
+         tt<-terms(formula,data=data)
+         mat<-attr(tt,"factors")
+         namfact<-colnames(mat)
+         nvar<-dim(mat)[1]
+         nfact<-dim(mat)[2]
+         possiP<-matrix(0,ncol=2,nrow=nfact)
+         dataF<-model.frame(formula,data,xlev=NULL)
+         namD<-names(dataF)
+         isF<-sapply(dataF, function(x) is.factor(x) || is.logical(x))
+         nlevel<-rep(0,nvar)
+         for(i in 1:nvar)
+         {
+             if(isF[i])
+             {
+                nlevel[i]<-length(table(dataF[[i]]))
+             }
+             else
+             {
+                nlevel[i]<-1
+             }
+         }
+         startp<-1+attr(tt, "intercept")
+         for(i in 1:nfact)
+         {
+             tmp1<-1
+             for(j in 1:nvar)
+             {
+                 if(mat[j,i]==1 && isF[j])
+                 {
+                    tmp1<-tmp1*(nlevel[j]-1)
+                 }
+             }
+             endp<-startp+tmp1-1
+             possiP[i,1]<-startp    
+             possiP[i,2]<-endp
+             startp<-endp+1
+         }
+         dimnames(possiP)<-list(namfact,c("Start","End"))
+
          #########################################################################################
          # misclassification
          #########################################################################################
@@ -139,7 +182,7 @@ function(formula,
  
          if(is.null(mcmc$tune))
          {
-            tune=1
+            tune=1.1
          }
          else
          {
@@ -176,8 +219,11 @@ function(formula,
                 beta<-fit0$beta
 		eta <- x %*% beta
 		v <- rep(0,nrec)
-		v[yobs==1]<-eta[yobs==1]-0.5
-		v[yobs==0]<-eta[yobs==0]+0.5
+		for(i in 1:nrec)
+		{
+		   if(yobs[i]==1)v[i]<-eta[i]-0.5
+		   if(yobs[i]==0)v[i]<-eta[i]+0.5
+		}
 		y<-yobs
 		theta<-log(3)
 	 }	
@@ -200,25 +246,26 @@ function(formula,
          nlink<-length(xlink)
          cpo<-rep(0,nrec)
          fsave     <- matrix(0, nrow=nsave, ncol=nlink)         
-	 thetasave <- matrix(0, nrow=nsave, ncol=p+3)
-	 randsave  <- matrix(0, nrow=nsave, ncol=nrec+1)
+	 thetasave <- matrix(0, nrow=nsave, ncol=(p+3))
+	 randsave  <- matrix(0, nrow=nsave, ncol=(nrec+1))
 
          #########################################################################################
          # working space
          #########################################################################################
 
-         #ntotal<-nrec
+         extra<-1
          ntotal<-nrec+nlink
 	 maxint<-2*(ntotal+3)+1
 	 maxend<-2*(ntotal+3)
-	 maxm<-log(0.0001)/log((ntotal+1)/(ntotal+2))
+	 maxm<-as.integer(log(0.0001)/log((ntotal+1)/(ntotal+2)))
 
          acrate<-rep(0,2)
 	 betac<-rep(0,p)
+	 clusts<-matrix(0,nrow=nrec,ncol=(nrec+1))	 
 	 endp<-rep(0,maxend)
 	 endp2<-rep(0,maxend)
 	 etan<-rep(0,nrec)
-	 fsavet<-rep(0,nlink)
+	 index<-rep(0,maxm)
 	 intcount<-rep(0,maxint)
 	 intcount2<-rep(0,maxint)
 	 intind<-rep(0,(nrec+1))
@@ -226,6 +273,7 @@ function(formula,
 	 iflag<-rep(0,p)
 	 intposso<-matrix(0,nrow=maxint,ncol=(nrec+1))
          intpossn<-matrix(0,nrow=maxint,ncol=(nrec+1))
+         limr<-matrix(0,nrow=nrec,ncol=2)
          lpsav<-rep(0,nrec)
          lpsavc<-rep(0,nrec)
 	 mass<-rep(0,maxint)
@@ -236,14 +284,11 @@ function(formula,
 	 ncluster<-nrec
 	 prob<-rep(0,maxint)
 	 proburn1<-rep(0,maxint)
-	 proburn2<-rep(0,maxint)
-	 proburn3<-rep(0,maxint)
-	 proburn4<-rep(0,maxint)
+	 s<-rep(0,nrec)	 
 	 urn<-rep(0,maxint)
 	 uvec<-rep(0,maxm)
 	 vvec<-rep(0,maxm)
 	 vnew<-rep(0,(nrec+1))
-	 vnew2<-rep(0,nrec)
 	 workm1<-matrix(0,nrow=p,ncol=p)
 	 workm2<-matrix(0,nrow=p,ncol=p)
 	 workmh1<-rep(0,(p*(p+1)/2))
@@ -258,8 +303,7 @@ function(formula,
  
          if(baseline=="logistic")
          {
-
-   	    foo <- .Fortran("csdpbinaryl2",
+   	    foo <- .Fortran("csdpbinaryl",
 	 	nrec      =as.integer(nrec),
 	 	p         =as.integer(p),
 		sens      =as.double(sens),
@@ -274,6 +318,7 @@ function(formula,
 		mcmcvec   =as.integer(mcmcvec),
 		nsave     =as.integer(nsave),
 		propv     =as.double(propv),
+		extra     =as.integer(extra),		
 		acrate    =as.double(acrate),
 		fsave     =as.double(fsave),
 		randsave  =as.double(randsave),
@@ -290,7 +335,8 @@ function(formula,
 		endp2     =as.double(endp2),
 		eta       =as.double(eta),
 		etan      =as.double(etan),
-		fsavet    =as.integer(fsavet),
+		clusts    =as.integer(clusts),		
+		index     =as.integer(index),
 		intcount  =as.integer(intcount),
 		intcount2 =as.integer(intcount2),
 		intind    =as.integer(intind),
@@ -298,6 +344,7 @@ function(formula,
 		iflag     =as.integer(iflag),
 		intposso  =as.integer(intposso),
 		intpossn  =as.integer(intpossn),
+		limr      =as.double(limr),		
 		lpsav     =as.integer(lpsav),
 		lpsavc    =as.integer(lpsavc),
 		maxint    =as.integer(maxint),
@@ -310,15 +357,12 @@ function(formula,
 		massurn4  =as.double(massurn4),
 		prob      =as.double(prob),
 		proburn1  =as.double(proburn1),
-		proburn2  =as.double(proburn2),
-		proburn3  =as.double(proburn3),
-		proburn4  =as.double(proburn4),
+		s         =as.integer(s),		
 		seed      =as.integer(seed),
                 urn       =as.integer(urn),
 		uvec      =as.double(uvec),
 		vvec      =as.double(vvec),
 		vnew      =as.double(vnew),
-		vnew2     =as.double(vnew2),
 		workm1    =as.double(workm1),
 		workm2    =as.double(workm2),
 		workmh1   =as.double(workmh1),
@@ -360,7 +404,7 @@ function(formula,
 	 z<-list(modelname=model.name,coefficients=coeff,acrate=foo$acrate,call=cl,
 	         prior=prior,mcmc=mcmcvec,state=state,save.state=save.state,nrec=foo$nrec,
 	         cpo=foo$cpo,p=p,nlink=nlink,xlink=xlink,x=x,
-	         ppar=prior$p,d=prior$d,baseline=baseline)
+	         ppar=prior$p,d=prior$d,baseline=baseline,possiP=possiP)
 	         
 	 cat("\n\n")
 
@@ -371,7 +415,7 @@ function(formula,
 
 ###                    
 ### Estimate the probability curve for a fitted semiparametric binary 
-### regression model (Falta optimizar. Probablemente codigo fortran).
+### regression model.
 ###
 ### Copyright: Alejandro Jara Vallejos, 2006
 ### Last modification: 21-08-2006.
@@ -509,10 +553,238 @@ predict.CSDPbinary<-function(object,xnew=NULL,hpd=TRUE, ...)
 
 
 ###
-### Tools for CSDPbinary: print, summary, plot
+### Tools for CSDPbinary: anova, print, summary, plot
 ###
 ### Copyright: Alejandro Jara Vallejos, 2006
-### Last modification: 21-08-2006.
+### Last modification: 15-12-2006.
+
+
+
+"anova.CSDPbinary"<-function(object, ...)
+{
+
+######################################################################################
+cregion<-function(x,probs=c(0.90,0.975))
+######################################################################################
+#  Function to compute a simultaneous credible region for a vector 
+#  parameter from the MCMC sample
+# 
+#  Reference: Besag, J., Green, P., Higdon, D. and Mengersen, K. (1995)
+#             Bayesian computation and stochastic systems (with Discussion)
+#             Statistical Science, vol. 10, 3 - 66, page 30
+#  and        Held, L. (2004) Simultaneous inference in risk assessment; a Bayesian 
+#             perspective In: COMPSTAT 2004, Proceedings in Computational 
+#             Statistics (J. Antoch, Ed.) 213 - 222, page 214
+#
+#  Arguments 
+#  sample : a data frame or matrix with sampled values (one column = one parameter).
+#  probs  : probabilities for which the credible regions are computed.
+######################################################################################
+{
+    #Basic information
+     nmonte<-dim(x)[1]
+     p<-dim(x)[2]
+     
+    #Ranks for each component
+     ranks <- apply(x, 2, rank, ties.method="first")
+     
+    #Compute the set S={max(nmonte+1-min r_i(t) , max r_i(t)): t=1,..,nmonte}
+     left <- nmonte + 1 - apply(ranks, 1, min)
+     right <- apply(ranks, 1, max)
+     S <- apply(cbind(left, right), 1, max)
+     S <- S[order(S)]
+    
+    #Compute the credible region
+     k <- floor(nmonte*probs)     
+     tstar <- S[k]
+     out<-list()
+     for(i in 1:length(tstar))
+     {
+        upelim <- x[ranks == tstar[i]]
+        lowlim <- x[ranks == nmonte + 1 - tstar[i]]    
+        out[[i]] <- rbind(lowlim, upelim)
+        rownames(out[[i]]) <- c("Lower", "Upper")
+        colnames(out[[i]]) <- colnames(x)
+     }
+     names(out) <- paste(probs)
+     return(out)
+}
+
+######################################################################################
+cint<-function(x,probs=c(0.90,0.975))
+######################################################################################
+#  Function to compute a credible interval from the MCMC sample
+#
+#  Arguments 
+#  sample : a data frame or matrix with sampled values (one column = one parameter).
+#  probs  : probabilities for which the credible regions are to be computed.
+######################################################################################
+{
+    #Compute the credible interval
+     delta<-(1-probs)/2
+     lprobs<-cbind(delta,probs+delta) 
+     out<-matrix(quantile(x,probs=lprobs),ncol=2)
+     colnames(out) <- c("Lower","Upper")
+     rownames(out) <- paste(probs)
+     return(out)
+}
+
+######################################################################################
+hnulleval<-function(mat,hnull)
+######################################################################################
+#  Evaluate H0
+#  AJV, 2006
+######################################################################################
+{
+     npar<-dim(mat)[2]   
+     lower<-rep(0,npar)
+     upper<-rep(0,npar)
+     for(i in 1:npar)
+     {
+        lower[i]<-mat[1,i]< hnull[i]
+        upper[i]<-mat[2,i]> hnull[i]
+     }
+     total<-lower+upper
+     out<-(sum(total==2) == npar)
+     return(out)
+}
+
+######################################################################################
+hnulleval2<-function(vec,hnull)
+######################################################################################
+#  Evaluate H0
+#  AJV, 2006
+######################################################################################
+{
+     lower<-vec[1]< hnull
+     upper<-vec[2]> hnull
+
+     total<-lower+upper
+     out<-(total==2)
+     return(out)
+}
+
+
+######################################################################################
+pcp<-function(x,hnull=NULL,precision=0.001,prob=0.95,digits=digits)
+######################################################################################
+#  Function to compute Pseudo Countour Probabilities (Region)
+#  AJV, 2006
+######################################################################################
+{
+    if(is.null(hnull))hnull<-rep(0,dim(x)[2])
+    if (dim(x)[2]!=length(hnull)) stop("Dimension of x and hnull must be equal!!")
+
+    probs <- seq(precision, 1-precision, by=precision)
+    neval <- length(probs)
+    probsf <- c(prob,probs)
+    cr <-  cregion(x,probs=probsf)
+
+    is.hnull <- hnulleval(cr[[2]],hnull)
+    if(is.hnull)
+    {
+       pval <- 1-precision
+    }   
+    else
+    {
+       is.hnull <- hnulleval(cr[[length(cr)]],hnull)
+       if (!is.hnull) 
+       {
+         pval <- precision
+       }  
+       else
+       {
+         is.hnull<-rep(0,neval+1)
+         for(i in 1:(neval+1))
+         {
+            is.hnull[i] <- hnulleval(cr[[i]],hnull)
+         }   
+         is.hnull <- is.hnull[-1]
+         first <- neval - sum(is.hnull) + 1
+         pval <- 1 - probs[first]
+       }
+    }
+    output <- list(cr=cr[[1]], prob=prob, pval=pval,hnull=hnull)
+    return(output)
+}
+
+
+######################################################################################
+pcp2<-function(x,hnull=NULL,precision=0.001,prob=0.95)
+######################################################################################
+#  Function to compute Pseudo Countour Probabilities (Interval)
+#  AJV, 2006
+######################################################################################
+{
+    if(is.null(hnull))hnull<-0
+    probs <- seq(precision, 1-precision, by=precision)
+    neval <- length(probs)
+    probsf <- c(prob,probs)
+    cr <-  cint(x,probs=probsf)
+
+    is.hnull <- hnulleval2(cr[2,],hnull)
+    if(is.hnull)
+    {
+       pval <- 1-precision
+    }   
+    else
+    {
+       is.hnull <- hnulleval2(cr[(neval+1),],hnull)
+       if (!is.hnull) 
+       {
+         pval <- precision
+       }  
+       else
+       {
+         is.hnull<-rep(0,neval+1)
+         for(i in 1:(neval+1))
+         {
+            is.hnull[i] <- hnulleval2(cr[i,],hnull)
+         }   
+         is.hnull <- is.hnull[-1]
+         first <- neval - sum(is.hnull) + 1
+         pval <- 1-probs[first]
+       }
+    }
+    output <- list(cr=cr[1,], prob=prob, pval=pval,hnull=hnull)
+    return(output)
+}
+
+######################################################################################
+######################################################################################
+######################################################################################
+
+    possiP<-object$possiP
+    nfact<-dim(possiP)[1]
+    P<-rep(0,nfact)
+    df<-rep(0,nfact)
+    
+    for(i in 1:nfact)
+    {
+        df[i]<-1
+        if((possiP[i,2]-possiP[i,1])>0)
+        { 
+           x<-matrix(object$save.state$thetasave[,possiP[i,1]:possiP[i,2]])
+           foo<-pcp(x=x) 
+           P[i]<-foo$pval
+           df[i]<-(possiP[i,2]-possiP[i,1])+1
+        }
+        else
+        {
+           x<-object$save.state$thetasave[,possiP[i,1]:possiP[i,2]]
+           foo<-pcp2(x=x) 
+           P[i]<-foo$pval
+        }
+    }
+
+    table <- data.frame(df,P) 
+    dimnames(table) <- list(rownames(possiP), c("Df","PsCP"))
+    structure(table, heading = c("Table of Pseudo Contour Probabilities\n", 
+        paste("Response:", deparse(formula(object)[[2]]))), class = c("anovaPsCP",
+        "data.frame"))
+}
+
+	
 
 
 "print.CSDPbinary"<-function (x, digits = max(3, getOption("digits") - 3), ...) 
